@@ -58,3 +58,24 @@ sub as_string {
 ###----------------------------------------------------------------###
 
 1;
+
+__END__
+
+=head1 DESCRIPTION
+
+Template::Alloy::Exception provides compatibility with Template::Exception
+and filters that require Template::Exception.
+
+=head1 TODO
+
+Document all of the methods.
+
+=head1 AUTHOR
+
+Paul Seamons <paul at seamons dot com>
+
+=head1 LICENSE
+
+This module may be distributed under the same terms as Perl itself.
+
+=cut
